;
.include "Header.inc"       ;Label und Werte Zuweisungen
.include "Hardware.inc"     ;Hardware Label  Zuweisungen
;
;Programmstart nach RESET  ( Interrupt Vektoren Tabelle )
;
.CSEG                       ;Was ab hier folgt kommt in den FLASH-Speicher
.ORG $0000                  ;Programm beginnt an der FLASH-Adresse 0x0000..
 rjmp  _reset               ;..mit der RESET-Vectortabelle

.include "IRQm88PA.inc"     ;Restliche Interrupt Vektortabelle

.ORG INT_VECTORS_SIZE       ;Programmadresse nach den ganzen IRQ-Vektoren
;
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;K Konstanten im Flash 
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;
_:
.DB 0;
;
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;I Erstinitialisierungen
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;
;Stapelzeiger initialisieren (fuer Unterprogramme und / oder Interrupts)
;
_reset:
 ldi  a,High(RAMEND)        ;RAMEND, also das Ende vom SRAM, ist in der.. 
 out  SPH,a                 ;..Definitions-datei festgelegt  
 ldi  a,Low (RAMEND)
 out  SPL,a
;
;Variablen initialisieren ( r0-r31 haben unbestimmten Inhalt )
;
 clr  null                  ;Register mit dem Inhalt Null
 clr  key_reg               ;Tasterentprellregister
 clr  flag_reg              ;Allgemeines Flaggenregister;
 clr  mwn                   ;Messwert..
 clr  mwl                   ;..erstmal..
 clr  mwh                   ;..auf Null

 ldi  xl,1
 clr  xh

 clr  z_adc_addition        ;ADC-Aditionszaehler auf eins setzen, um 256..
;
;Ports Erstkonfiguration
; 
 sbi  PLA_PORT,led.pla      ;PraogrammLaufAnzeige-LED aus ( Vorerst Pullup )
 sbi  PLA_DDR, led.pla      ;PORT-Bit als Ausgang f. PLA-LED ( Jetzt High )
 
 sbi  PLA_PORT,led.err      ;Error-LED ausschalten ( Vorerst Pullup )
 sbi  PLA_DDR, led.err      ;PORT-Bit als Ausgang f. Error-LED ( Jetzt High )
  
 sbi  PWM2A_DDR,OC2A         ;Portpinn als normalen Ausgang  freigeben..
 sbi  PWM2B_DDR,OC2B         ;..PWM-Portpinn als Ausgang  freigeben

 sbi  LED_PORT,led.ok       ;OK-LED ausschalten ( Vorerst Pullup )
 sbi  LED_DDR, led.ok       ;PORT-Bit als Ausgang f. OK-LED ( Jetzt High )

 sbi  KEY_PORT,key.bit      ;Pullup f. Taster einschalten
;
; USART Baudrate einstellen
;
 ldi  a,#ubrr               ;f. UBRRH bei 38400 Baud mit 1,8432Hz Quarz
 clr  b                     ;f. UBBRL bei 38400 Baud mit 1,8432Hz Quarz
 sts  UBRR0H,b              ;Baudrate 38400 mit einem...
 sts  UBRR0L,a              ;...1,8432Hz Quarz
;
;USART Sendeschieberegisterinterrupt und Sender ( Transmitter ) freigeben
;
 ldi  a,1<<TXCIE0|1<<TXEN0  ;USART Sendeschieberegisterinterrupt freigeben...
 sts  UCSR0B,a              ;...und USART-Sendeeinheit einschalten
;
;USART Format ( 8,N,1 ) ist bereits nach dem Reset voreingestellt
;
;
;Timer0 Overflow Interrupt initialisieren
;
 ldi  a,1<<TOIE0            ;Timer0 Overflow Interrupt freigeben...
 sts  TIMSK0,a              ;...
 ldi  a,1<<CS02             ;Timer0 Vorteiler => 256...
 out  TCCR0B,a              ;...
;
;Timer/Counter2: Phase Correkt PWM-Mode ( 5 ) initialisieren, NON-INVERT, 
;ohne Vorteiler. Clear OC2B on Compare Match upcounting, set OC2B on Compare 
;Match downcounting. OC2B ist der PWM-Pinn, da OCR2A ja den TOP-Wert enthlt.
;Der OC2A-Pinn ist ein normaler Output und bestimmt die Drehrichtung.
;TOV2-ISR bei BOTTOM.
;
 ldi  a,#pcpwm_freq         ;10kHz an OC2B einstellen,..
 sts  OCR2A,a               ;..
 lsr  a
 sts  OCR2B,a

 ldi  a,NINV0B|1<<WGM00     ;Phase Correkt PWM-Mode 5..
 sts  TCCR2A,a              ;..Upcouting clear OC0B on Compare Match, Downcount-..
 ldi  a,1<<CS00|1<<WGM02    ;...ting set OC0B on Compare Match, ohne Vor-..
 sts  TCCR2B,a              ;..teiler, Timer/Counter0 starten und..
 sbi  PWM2B_DDR,OC2B         ;..PWM-Portpinn als Ausgang  freigeben
;
;Timer2 Overflow Interrupt initialisieren
;
 ;ldi  a,1<<TOIE2            ;Timer2 Overflow Interrupt freigeben...
 ;sts  TIMSK2,a              ;...
 ;ldi  a,1<<CS22|1<<CS21|1<<CS20;Timer2 Vorteiler => 1024...
 ;sts  TCCR2B,a              ;...
;
;ADC Erstwandlung im Free Running Mode. Danach Einzelwandlungen mit ADC-ISR.
;10-Bit, Aref=AVcc, 115200 Hz, digitale Eingangspuffer alle abschalten 
;
 ldi  a,$3F                 ;Alle digitalen Eingangspuffer abschalten..
 sts  DIDR0,a               ;..(PC0-5), um die Leistungsaufnahme zu reduzieren

 ldi  a,1<<REFS0            ;Aref = AVcc.. 
 sts  ADMUX,a               ;..mit 100nF am Aref-Pin

 ldi  a,1<<ADEN|1<<ADSC|1<<ADATE|1<<ADIE|1<<ADPS2;ADC Freigeben, Erstwandlung starten,..
 sts  ADCSRA,a              ;..Free Running Mode und Vorteiler auf 16 

 sei                        ;Globale Interruptfreigabe im CPU-Statusregister
;
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;H Hauptprogrammschleife
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;
_main:
 lds   a,ADCSRA             ;Nur wenn ADATE und ADIF gesetzt sind, dann ist.. 
 sbrc  a,ADATE              ;..die Erstwandlung abgeschlossen. Dieser..
 sbrs  a,ADIF               ;..ADC-Wert ist unbrauchbar und wird verworfen.
 rjmp  _mwert_check 
 ldi   a,1<<ADEN|1<<ADSC|1<<ADIF|1<<ADIE|1<<ADPS2;Einzelwandlung mit ADC-ISR..
 sts   ADCSRA,a             ;..starten

 ;sbrs key_reg,key.actuated  ;Taster betaetigt und entprellt?..
 ;rjmp _idle                 ;Programmlaufschleife
_mwert_check: 
 sbrs flag_reg,mwert.rdy    ;ADC-Mittelwertbildung fertig = Ueberspringen
 rjmp _mwert_busy           ;ADC-Mittelwertbildung ist nicht fertig

 cli
 ldi  zl, LOW(_adc_werte)   ;SRAM-Anfangsadresse der ADC-Werte.. 
 ldi  zh,HIGH(_adc_werte)   ;..holen und..
 ld   a,z+                  ;..Bytes..
 ld   b,z                   ;..laden
 sei

;cli
;TESTZWECKE ####################################################################
 rcall _warte_udre          ;TESTZWECKE ########################################
 ;lds  b,ADCH
 sts  UDR0,b                ;TESTZWECKE ########################################
 rcall _warte_udre          ;TESTZWECKE ########################################
 ;lds  a,ADCL
 sts  UDR0,a                ;TESTZWECKE ########################################
 rcall _warte_udre          ;TESTZWECKE ########################################
 sts  UDR0,z_adc_addition   ;TESTZWECKE ########################################
;TESTZWECKE ####################################################################
 ;sei

 cbr  flag_reg,1<<mwert.rdy ;Flag ADC-Mittelwertsbildung fertig ruecksetzen

_mwert_busy:
 sbi  LED_PIN,led.ok        ;OK-LED toggeln
 cbr  key_reg,1<<key.actuated;Taster wieder aktivieren
 rjmp _idle                 ;Programmlaufschleife
 
;TESTZWECKE ####################################################################
 ;rcall _warte_udre          ;TESTZWECKE ########################################
 ;ldi  a,13                  ;CR -> Carriage Return ( Wagenruecklauf )
 ;sts  UDR0,a                ;TESTZWECKE ########################################
 ;rcall _warte_udre          ;TESTZWECKE ########################################
 ;ldi  a,10                  ;LF -> Line Feed ( Zeilenvorschub )
 ;sts  UDR0,a                ;TESTZWECKE ########################################
 ;rcall _warte_udre          ;TESTZWECKE ########################################
 ;sts  UDR0,b                ;Korrigierter PWM-Wert
;TESTZWECKE ####################################################################

;
;ProgrammLaufAnzeige (PLA) in Gang halten
;
_idle: 
 sbr  flag_reg,1<<pla.flg   ;Programmlaufanzeige-Flag quitieren
 
 rjmp _main                 ;Sprung zum Anfang des Hauptprogramms ( Mainloop )
; 
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;U Beginn der verwendeten Unterprogramme
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;
.include "UP.inc";Unterprogramme
;
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;Q Beginn der verwendeten Interrupt Service Routinen ( ISR )
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;
.include "ISR.inc";Interrupt Service Routinen
;
;DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
;DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
;D Daten im SRAM ablegen
;DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
;DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
;
.DSEG

_adc_werte:
.BYTE #adc_anz          ;Anzahl der Eingelesenen ADC-Werte
 
.EXIT                       ;Ende des Quelltextes
