;
.include "Header.inc"       ;Label und Werte Zuweisungen
.include "Hardware.inc"     ;Hardware Label  Zuweisungen
;
;Programmstart nach RESET  ( Interrupt Vektoren Tabelle )
;
.CSEG                       ;Was ab hier folgt kommt in den FLASH-Speicher
.ORG $0000                  ;Programm beginnt an der FLASH-Adresse 0x0000..
 rjmp  _reset               ;..mit der RESET-Vectortabelle

.include "IRQm88PA.inc"     ;Restliche Interrupt Vektortabelle

.ORG INT_VECTORS_SIZE       ;Programmadresse nach den ganzen IRQ-Vektoren
;
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;K Konstanten im Flash 
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;
;.DB 
;
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;I Erstinitialisierungen
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;
;Stapelzeiger initialisieren (fuer Unterprogramme und / oder Interrupts)
;
_reset:
 ldi  a,High(RAMEND)        ;RAMEND, also das Ende vom SRAM, ist in der.. 
 out  SPH,a                 ;..Definitions-datei festgelegt  
 ldi  a,Low (RAMEND)
 out  SPL,a
;
;Variablen initialisieren ( r0-r31 haben unbestimmten Inhalt )
;
 clr  null                  ;Register mit dem Inhalt Null belegen
 clr  key_reg               ;Tasterentprellregister
 clr  flag_reg				;Allgemeines Flaggen-Register ruecksetzen
 clr  r_puls				;Register f. die Pulsdauer
 
 mov  eins,null				;Ein unteres Register ( r0-r15 ) mit..
 inc  eins					;..Eins belegen
;
;Ports Erstkonfiguration
; 
 ser  a						;Port..
 out  OUT_DDR,a				;..komplett als Ausgang

 sbi  KEY_PORT,key.pin		;Pullup fuer den Taster einschalten
 
 ldi  a,$FF					;Drehgeberport zunaechst alles mit Pullup..
 out  DRG_PORT,a			;..versehen..
 cbi  DRG_PORT,drg.a		;..dann Spur A und..
 cbi  DRG_PORT,drg.b		;..Spur B hochohmig, da externer Pullup vorhanden.
 sbi  DRG_DDR,drg.stop		;LED Drehgebertaster wurde gedrueckt erstmal aus.

 sbi  LED_PORT,led.in_1		;LED Richtung erstmal Pullup
 sbi  LED_DDR,led.in_1		;LED Richtung ausschalten ( High )
 sbi  LED_PORT,led.in_2		;LED /Richtung erstmal Pullup
 sbi  LED_DDR,led.in_2		;LED /Richtung ausschalten ( High )
 ;; sbi  DRG_PORT,DRGB.b		;Drehgeber SpurB (  Pullup )
 
 sbi  PLA_DDR, pla.led      ;PORT-Bit als Ausgang f. PLA-LED ( Jetzt High )
;
;Timer/Counter0 Overflow Interrupt initialisieren. Ca. alle 800s
;
 ldi  a,1<<TOIE0            ;Timer0 Overflow Interrupt freigeben...
 sts  TIMSK0,a              ;...
 ldi  a,1<<CS01				;Timer0 Vorteiler => 8...
 out  TCCR0B,a              ;...
;
;Timer2 Overflow Interrupt initialisieren
;
 ;ldi  a,1<<TOIE2            ;Timer2 Overflow Interrupt freigeben...
 ;:sts  TIMSK2,a              ;...
 ;ldi  a,1<<CS22|1<<CS21|1<<CS20;Timer2 Vorteiler => 1024...
 ;sts  TCCR2B,a              ;...
;
;Timer/Counter2: Phase Correkt PWM-Mode ( 5 ) initialisieren, NON-INVERT, 
;ohne Vorteiler. Clear OC2B on Compare Match upcounting, set OC2B on Compare 
;Match downcounting. OC2B ist der PWM-Pinn, da OCR2A ja den TOP-Wert enthlt.
;Der OC2A-Pinn ist ein normaler Output und bestimmt die Drehrichtung.
;TOV2-ISR bei BOTTOM.
;
 ldi  a,pcpwm_freq          ;10kHz an OC2B einstellen,..
 sts  OCR2A,a	            ;..
 ;lsr  a
 sts  OCR2B,null

 ldi  a,NINV0B|1<<WGM00     ;Phase Correkt PWM-Mode 5..
 sts  TCCR2A,a              ;..Upcouting clear OC0B on Compare Match, Downcount-..
 ldi  a,1<<CS00|1<<WGM02    ;...ting set OC0B on Compare Match, ohne Vor-..
 sts  TCCR2B,a              ;..teiler, Timer/Counter0 starten und..
 sbi  PWM2B_DDR,pwm2b.oc2b  ;..PWM-Portpinn als Ausgang  freigeben
 
 sei                        ;Globale Interruptfreigabe im CPU-Statusregister
;
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;H Hauptprogrammschleife
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;
_main:
 sbr  flag_reg,1<<pla.flg   ;Programmlaufanzeige-Flag quitieren
;
;Handencoder ( Drehgeber ) bearbeiten. 
;
_handencoder:
 sbrc key_reg,key.actuated  ;Drehgeber-Taster betaetigt?
 rjmp _encoder_taster	    ;JA -> Motor stillsetzen
 sbrc flag_reg,drg.links	;Drehgeber links  gedreht?
 rjmp _encoder_links		;JA 
 sbrc flag_reg,drg.rechts	;Drehgeber rechts gedreht?
 rjmp _encoder_rechts		;JA
 rjmp _main					;Hauptprogrammschleife
;
;Motor sillsetzen
;
_encoder_taster: 
 cbr  key_reg,1<<key.actuated;Tastererkennung wieder freigeben
 cbr  flag_reg,1<<mot.rechts;Motordrehrichtung Reset
 cbr  flag_reg,1<<mot.links ;Motordrehrichtung Reset
 cbi  LED_PORT,led.in_1		;H-Bruecke bremsen
 cbi  LED_PORT,led.in_2		;H-Bruecke bremsen
 cbi  DRG_PORT,drg.stop		;LED Drehgebertaster wurde betaetigt einschalten
 cbr  flag_reg,1<<drg.links ;Flag fuer Drehgeber links  gedreht Reset
 cbr  flag_reg,1<<drg.rechts;Flag fuer Drehgeber rechts gedreht Reset
 
 clr  r_puls				;Pulsdauer auf Null setzen ( kein PWM ) und dies..
 sts  PULS,r_puls           ;..an das SFR-Register OCR2B uebergeben.
 clr  eins					;Register neu mit Eins..
 inc  eins					;..initialisieren

 out  OUT_PORT,null			;LED-Anzeigeband = Null
 rjmp _main					;Hauptprogrammschleife
;
;Handencoder Spur A ( Rechtsdrehen ) bearbeiten
;
_encoder_rechts: 
 cbr   flag_reg,1<<drg.rechts;Flag fuer Drehgeber rechts gedreht Reset
 sbrs flag_reg,mot.links	;War die erste Drehgeberrichtung links?
 sbr  flag_reg,1<<mot.rechts;NEIN -> Motor soll rechts drehen
 sbi  DRG_PORT,drg.stop		;LED Drehgebertaster wurde betaetigt ausschalten
 sbrs flag_reg,mot.rechts	;Motordrehrichtung rechts?
 rjmp _puls_verringern		;NEIN -> Pulsdauer verringern
_puls_erhoehen: 
 mov  a,r_puls				;Register f. die Pulsdauer f. Vergleich kopieren..
 cpi  a,pcpwm_freq			;..maximale Pulsdauer erreicht? ( Ausgang High? )
 brcc _skip					;JA -> Pulsdauer nicht mehr incrementieren 
 inc  r_puls				;Pulsdauer erhoehen und..
_skip:
 sts  PULS,r_puls           ;..PWM Pulsdauer anpassen
 rjmp _led_band 			;LED-Anzeigeband bearbeiten
_puls_verringern:
 cpse null,r_puls			;Pulsdauer minimum? Falls ja nicht mehr decremieren
 dec  r_puls				;Register f. die Pulsdauer
 sts  PULS,r_puls           ;PWM Pulsdauer einstellen
 rjmp _led_band				;LED-Anzeigeband bearbeiten
;
;Handencoder Spur B ( Linksdrehen ) bearbeiten
;
_encoder_links:
 cbr   flag_reg,1<<drg.links;Flag f. Drehgeber links gedreht Reset
 sbrs  flag_reg,mot.rechts	;War die erste Drehgeberrichtung rechts?
 sbr   flag_reg,1<<mot.links;NEIN -> Motor soll links drehen
 sbi  DRG_PORT,drg.stop		;LED Drehgebertaster wurde betaetigt ausschalten
 sbrs flag_reg,mot.links	;Motordrehrichtung links?
 rjmp _puls_verringern		;NEIN -> Pulsdauer verringern
 rjmp _puls_erhoehen		;Pulsdauer erhoehen
;
;LED-Anzeigeband bearbeiten
;
_led_band:
 sbrs  flag_reg,mot.rechts	;Motordrehrichtung rechts?
 rjmp  _led_links			;NEIN -> LED-Band links laufen lassen
;
;LED-Anzeige rechts wandern lassen
;
_led_rechts:
 sbi  LED_PORT,led.in_1		;H-Bruecke Drehrichtung -> Rechts einstellen
 cbi  LED_PORT,led.in_2		;H-Bruecke Drehrichtung -> Rechts einstellen
 cpse null,r_puls			;Pulsdauer minimum? Falls ja Min anzeigen
 rjmp _led_rechts_max		;LED-Anzeige rechts wandern lassen
 ldi  a,$01					;Bit0 auf High setzen und hiermit..
 out  OUT_PORT,a			;..Minimum anzeigen
 rjmp _main					;Hauptprogrammschleife
_led_rechts_max:
 mov  a,r_puls				;Register f. die Pulsdauer f. Vergleich kopieren..
 cpi  a,pcpwm_freq			;..maximale Pulsdauer erreicht?
 brne _led_rechts_next		;NEIN -> LED rechts wandern lassen
 ldi  a,$80					;Bit7 auf High setzen und hiermit..
 out  OUT_PORT,a			;..Maximum anzeigen
 rjmp _main					;Hauptprogrammschleife
_led_rechts_next:
 in   a,OUT_PORT			;Momentanen Portzustand einlesen
 clc						;Carry-Flag loeschen
 tst  a					    ;Hat das Register den Wert Null?
 brne _ror_eins				;NEIN -> Register rechts rotieren
 sec						;Carry-Flag setzen  
_ror_eins:
 ror  a
 out  OUT_PORT,a
 rjmp _main					;Hauptprogrammschleife
;
;LED-Anzeige links wandern lassen
;
_led_links: 
 cbi  LED_PORT,led.in_1		;H-Bruecke Drehrichtung -> Links einstellen
 sbi  LED_PORT,led.in_2		;H-Bruecke Drehrichtung -> Links einstellen
 cpse null,r_puls			;Pulsdauer minimum? Falls ja Min anzeigen
 rjmp _led_links_max		;Pulsdauer maximum ueberpruefen
_led_min:
 ldi  a,$01					;Bit0 auf High setzen und hiermit..
 out  OUT_PORT,a			;..Minimum anzeigen
 rjmp _main					;Hauptprogrammschleifemov  a,r_puls				;Register f. die Pulsdauer f. Vergleich kopieren..
_led_links_max: 
 mov  a,r_puls				;Register f. die Pulsdauer f. Vergleich kopieren..
 cpi  a,pcpwm_freq			;..maximale Pulsdauer erreicht?
 brcs _led_links_next		;NEIN -> LED links wandern lassen
 ldi  a,$80					;Bit7 auf High setzen und hiermit..
 out  OUT_PORT,a			;..Maximum anzeigen
 rjmp _main					;Hauptprogrammschleife
_led_links_next:
 in   a,OUT_PORT			;Momentanen Portzustand einlesen
 clc						;Carry-Flag loeschen
 tst  a					    ;Hat das Register den Wert Null?
 brne _rol_eins				;NEIN -> Register rechts rotieren
 sec						;Carry-Flag setzen  
_rol_eins:
 rol  a
 out  OUT_PORT,a
 rjmp _main					;Hauptprogrammschleife
 
; in   a,OUT_PORT			;Momentanen Portzustand einlesen
 ;rol  a
 ;brcs _led_min
 ;sec						;Carry-Flag setzen  
 ;out  OUT_PORT,a
 ;rjmp _main					;Hauptprogrammschleife; 
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;U Beginn der verwendeten Unterprogramme
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;
.include "UP.inc";Unterprogramme
;
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;Q Beginn der verwendeten Interrupt Service Routinen ( ISR )
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;
.include "ISR.inc";Interrupt Service Routinen
;
;DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
;DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
;D Daten im SRAM ablegen
;DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
;DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
;
.DSEG
;_adc_werte:
;.BYTE  adc_azhl          ;Anzahl der Eingelesenen ADC-Werte
 
.EXIT                       ;Ende des Quelltextes
 