;
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
; Headerdatei zu Handencoder
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;Der ATmega88PA laeuft mit 1MHz. ( Auslieferungszustand )
;
;FUSES Extendet $F9, High $DF, Low $62. ( Auslieferungszustand )
;
;Mit einem "Handencoder" Typ: Panasonic EVEQ DBRL416B wird per PWM ein DC-Motor
;gesteuert. Dabei wird die Drehrichtung und Geschwindigkeit mit dem Encoder be-
;stimmt. Links drehen laesst den Motor links drehen und je nach Drehdauer in 
;die gleiche Richtung, wird die Drehzahl bis zum Maximum erhoeht. Drehen in die 
;andere Richtung verringert die Drehzahl und geht bis zum Stillstand ( Null ).
; 
;PortB zeigt mit einem LED-Lauflicht die Drehrichtung und die Minimalen und
;Maximalen PWM-Werte an. ( Max = Bit7; Min = Bit0 ). Durch Low-Zustand zeigen
;Pin6 die Drehrichtung rechts an und Pin26 Stopp an. Pin25 blinkt und zeigt
;damit den Programmlauf an. ( PLA = ProgrammLaufAnzeige ) 
;
;Ein Druck auf den Encoderknopf laesst den Motor ueber die Motorbremsung 
;( L298N-Platine ) zum Stillstand kommen ( Stopp ) und eine andere Drehrichtung 
;kann gewaehlt werden.
;
;CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
; Einbinden der Controllerspezifischen Definitionsdatei
;CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
;
.NOLIST                 ;List-Output ausschalten
.INCLUDE "m88PAdef.inc" ;AVR-Definitionsdatei einbinden
.LIST                   ;List-Output wieder einschalten
;
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
; Arbeitsregister umbenennen
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;
;.DEF        = r0  ;
;.DEF        = r1  ;
.DEF null			   = r2  ;Register mit dem Inhalt Null
;.DEF 	     = r3  ;
.DEF eins		       = r4  ;Register mit dem Inhalt 
;.DEF        = r5  ;
.DEF ovf0_zlr		   = r6  ;Timer0 Overflow-Zaehler
.DEF r_puls			   = r7  ;Register fuer den Pulsdauer-Wert
;.DEF		 = r8  ;
;.DEF        = r9  ;
;.DEF		 = r10 ; 
;.DEF        = r11 ;
;.DEF        = r12 ;
;.DEF        = r13 ;
;.DEF        = r14 ;
.DEF s_sreg            = r15  ;Zum Sichern des Statusregisters ( Nur 1 Takt )

.DEF a                 = r16  ;Allgemeines Arbeitsregister
.DEF b                 = r17  ;Allgemeines Arbeitsregister
.DEF key_reg	       = r19  ;Register f. die Tasterbearbeitung
.DEF drg_merker        = r20  ;Drehgeberwert neu fuer Weiterverarbeitung.
.DEF ia	               = r21  ;Interrupt Arbeitsregister ia ( Nur in ISR genutzt )
;.DEF        = r22  ;
.DEF flag_reg	       = r23  ;Allgemeines Flaggen-Register
;.DEF        = r24  ;
;.DEF        = r25  ;
.DEF xl                = r26  ;16-Bit-Zaehler fuer..
.DEF xh                = r27  ;..SK6812 Reset-Zeit
.DEF yl                = r28  ;16-Bit.. 
.DEF yh                = r29  ;..Zeitschinder
.DEF zl                = r30  ;Verwendung als Z-Zeiger, also wie bereits in der..
.DEF zh                = r31  ;..Definitionsdatei von ATMEL angegeben
;
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
; Konstanten als Symbole
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;
.EQU pcpwm_freq	    = 92		  ;Zhlumfang*2 = Auflsung ) f. 10kHz
.EQU NINV0A         = 1<<COM0A1|0<<COM0A0;OC0A (Non-Inverting Mode)
.EQU INV0A          = 1<<COM0A1|1<<COM0A0;OC0A (Interting Mode)
.EQU TOGG0A         = 0<<COM0A1|1<<COM0A0;OC0A (Toggle Mode nur OC0A moeglich)

.EQU NINV0B         = 1<<COM0B1|0<<COM0B0;OC0B (Non-Inverting Mode)
.EQU INV0B          = 1<<COM0B1|1<<COM0B0;OC0B (Interting Mode)
.EQU PULS           = OCR2B		  ;Pulsdauer

.EQU #ovf0_zlr		   = 7	      ;Wert f. Timer0 Overflow-Zaehler
;.EQU 		   = 7	      ;
;
;Drehgeber 
;
.EQU drg_msk		 = 0b00000011 ;Drehgebermaske fuer Spur A & B
.EQU drg_cpi         = 0b00001111 ;Maske fuer Drehgeberwerte vergleichen
.EQU drg_r1			 = 0b00000010 ;1.Vergleichswert f. Rechts
.EQU drg_r2			 = 0b00001101 ;2.Vergleichswert f. Rechts
.EQU drg_l1			 = 0b00000001 ;1.Vergleichswert f. Links 
.EQU drg_l2			 = 0b00001110 ;2.Vergleichswert f. Links 
;
;Bitbezeichner fuer das allgemeine Flaggenregister flag_reg
;
;.equ			= 0	;
.equ  mot.rechts	= 1	;Motordrehrichtung Rechts = gesetzt
.equ  mot.links     = 2	;Motordrehrichtung Links  = gesetzt
;.equ		   = 3		;
.equ  pla.flg		= 4	;Programmlaufanzeige-Quitierung
.equ  drg.links	 	= 5	;Drehgeber wurde links  gedreht
.equ  drg.rechts    = 6	;Drehgeber wurde rechts gedreht
;.equ		    = 7	; 
;
;Bitbezeichner fuer das Tasterentprellregister key_reg
;
;.equ			= 0	;Taster..
;.equ			= 1	;..Prellzaehler
.equ  key.unactuated	= 2	;Taster entprellt unbetaetigt
.equ  key.actuated		= 3	;Taster entprellt betaetigt
;.equ			= 4	;
;.equ	        = 5	;
;.equ	        = 6	; 
;.equ		    = 7	; 